<?php

namespace Drupal\iconset\Element;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Element\RenderElement;
use Drupal\iconset\IconsetManagerInterface;
use Drupal\iconset\Exception\IconNotFoundException;

/**
 * Render a single icon from an iconset.
 *
 * Properties:
 * - #iconset: The iconset identifier for the set the icon belongs to.
 * - #icon: The ID of the icon to render.
 * - #alt_text: Alt text to use for accessibility or if browser fails to render.
 *
 * Usage example:
 * @code
 * $build['example_icon'] = [
 *   '#type' => 'iconset_icon',
 *   '#iconset' => 'fontawesome_brands',
 *   '#icon' => 'facebook_square',
 *   '#widith' => 40,
 *   '#height' => 40,
 *   '#alt_text => t('Facebook'),
 *   '#attributes => [
 *     'class' => ['icon--large'],
 *   ],
 * ];
 * @endcode
 *
 * @RenderElement("iconset_icon")
 */
class IconsetIcon extends RenderElement implements ContainerFactoryPluginInterface {

  /**
   * The iconset manager.
   *
   * @var \Drupal\iconset\IconsetManagerInterface
   */
  protected $iconsetManager;

  /**
   * Creates a new instance of the IconsetIcon render element.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\iconset\IconsetManagerInterface $iconset_manager
   *   The iconset manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, IconsetManagerInterface $iconset_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->iconsetManager = $iconset_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.iconset')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      '#iconset' => '',
      '#icon' => '',
      '#alt_text' => '',
      '#attributes' => [],
      '#decorative' => FALSE,
      '#pre_render' => [static::class . '::preRender'],
    ];
  }

  /**
   * Transform the icon information into a renderably array of theme elements.
   *
   * @param array $element
   *   The element to alter the rendering content for.
   *
   * @return array
   *   Renderable array to render the icon.
   */
  public static function preRender(array $element) {
    if (!(empty($element['#iconset']) || empty($element['#icon']))) {
      try {
        $options = [];
        $options['decorative'] = $element['#decorative'];

        if ($element['#alt_text']) {
          $options['alt'] = trim($element['#alt_text']);
        }

        // Though most icon types (images) support width and height as
        // HTML tag attributes, we push this information as options, and leave
        // it to the iconset builder to determine how to use this information.
        //
        // This allows more flexibility with implementations.
        if (!empty($element['#width'])) {
          $options['width'] = intval($element['#width']);
        }
        if (!empty($element['#height'])) {
          $options['height'] = intval($element['#height']);
        }

        $content = $this->iconsetManager
          ->createInstance($element['#iconset'])
          ->build($element['#icon'], $options);

        if ($content) {
          if (!empty($content['#attributes']['class'])) {
            $element['#attributes'] += ['class' => []];
            $element['#attributes']['class'] = array_merge($content['#attributes']['class'], $element['#attributes']['class']);
          }
          $content['#attributes'] = $element['#attributes'] + $content['#attributes'];

          return $content;
        }
      }
      catch (PluginException | IconNotFoundException $e) {
        // @todo Render a default, or no image icon placeholder as a fallback.
      }
    }

    return [];
  }

}
